/*
   Do not modify this file.
*/

/**
   This simulates a first-in-first-out (stack) buffer.
*/
public class Buffer
{
   protected final int SIZE;
   protected final char[] buf;
   protected int currentIndex;

   /**
      Create a half filled buffer.

      @param c1  the left bracket for the string representation of the buffer
      @param c4  the right bracket for the string representation of the buffer
      @param SIZE  the size of the buffer
   */
   Buffer(char c1, char c4, final int SIZE)
   {
      this(SIZE/2, c1, '0', '_', c4, SIZE);
   }


   /**
      Initialize a partially filled buffer.

      @param currentIndex  initially fill the buffer up to this index
      @param c1  the left bracket for the string representation of the buffer
      @param c4  the right bracket for the string representation of the buffer
      @param SIZE  the size of the buffer
   */
   Buffer(int currentIndex, char c1, char c4, final int SIZE)
   {
      this(currentIndex, c1, '0', '_', c4, 80);
   }


   /**
      Initialize a partially filled buffer.

      @param currentIndex  initially fill the buffer up to this index
      @param c1  the left bracket for the string representation of the buffer
      @param c2  the char for the filled part of the buffer
      @param c3  the char for the empty part of the buffer
      @param c4  the right bracket for the string representation of the buffer
      @param SIZE  the size of the buffer
   */
   Buffer(int currentIndex, char c1, char c2, char c3, char c4, final int SIZE)
   {
      // currentIndex == 0 initializes an empty buffer
      // currentIndex == SIZE initializes a full buffer
      if (currentIndex < 0 || currentIndex > SIZE)
         throw new Error("illegal buffer initialization");

      this.SIZE = SIZE;
      this.buf = new char[SIZE + 2];
      this.currentIndex = currentIndex;
      this.buf[0] = c1;         //mark the beginning of the buffer
      for (int i = 1; i <= currentIndex; ++i)
      {
         this.buf[i] = c2;     // mark the full part of the buffer
      }
      for (int i = currentIndex + 1; i <= SIZE; ++i)
      {
         this.buf[i] = c3;      // mark the empty part of the buffer
      }
      this.buf[SIZE + 1] = c4;  // mark the end of the buffer
   }


   /**
      Insert the given character at the next available buffer location.

      Every time we do an operation on this buffer, it prints out its state.

      @param c  character to put in the buffer
   */
   public void put(final char c)
   {
      if (currentIndex == SIZE)
         throw new Error("buffer is full");

      ++currentIndex;
      Thread.yield(); // force race conditions to happen more often
      buf[currentIndex] = c;
      System.out.println(this);
   }


   /**
      Remove from the buffer the character at the buffer's current index.
      Mark the newly emptied spot in the buffer with the given character.
      Return the character that was removed from the buffer.

      Every time we do an operation on this buffer, it prints out its state.

      @param c  character to use to mark the newly emptied position in the buffer
      @return the character removed from the buffer
   */
   public char get(final char c)
   {
      if (currentIndex == 0)
         throw new Error("buffer is empty");

      final char result = buf[currentIndex];
      buf[currentIndex] = c;
      Thread.yield(); // force race conditions to happen more often
      --currentIndex;
      System.out.println(this);
      return result;
   }


   public boolean isFull()
   {
      return currentIndex == SIZE;
   }


   public boolean isEmpty()
   {
      return currentIndex == 0;
   }


   /**
      Create a String representation of this buffer showing the
      buffer's contents in the first line and the current position
      of the buffer's index in the second line.
   */
   @Override
   public String toString()
   {
      String result = "";
      // first line
      for (int i = 0; i < SIZE + 2; ++i)
      {
         result += buf[i];
      }
      result += "\n";
      // second line
      for (int i = 0; i < this.currentIndex; ++i)
      {
         result += " ";
      }
      result += "^\n";
      return result;
   }
}
