/*


*/

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.TimeUnit;
import java.util.Optional; // for the return value of get()

/**
   This simulates a first-in-first-out (stack) buffer.
*/
public class Buffer_TimeOut extends Buffer
{
   private final Lock lock;
   private final Condition emptyCondition;
   private boolean empty;
   private final Condition fullCondition;
   private boolean full;

   /**
      Create a half filled buffer.

      @param c1  the left bracket for the string representation of the buffer
      @param c4  the right bracket for the string representation of the buffer
      @param SIZE  the size of the buffer
   */
   Buffer_TimeOut(char c1, char c4, final int SIZE)
   {
      this(SIZE/2, c1, '0', '_', c4, SIZE);
   }


   /**
      Initialize a partially filled buffer.

      @param currentIndex  initially fill the buffer up to this index
      @param c1  the left bracket for the string representation of the buffer
      @param c4  the right bracket for the string representation of the buffer
      @param SIZE  the size of the buffer
   */
   Buffer_TimeOut(int currentIndex, char c1, char c4, final int SIZE)
   {
      this(currentIndex, c1, '0', '_', c4, 80);
   }


   /**
      Create a partially filled buffer with a mutual exclusion lock,
      a condition variable that prevents buffer underflow, and
      a condition variable that prevents buffer overflow.

      @param currentIndex  initially fill the buffer up to this index
      @param c1  the left bracket for the string representation of the buffer
      @param c2  the char for the filled part of the buffer
      @param c3  the char for the empty part of the buffer
      @param c4  the right bracket for the string representation of the buffer
      @param SIZE  the size of the buffer
   */
   Buffer_TimeOut(int currentIndex, char c1, char c2, char c3, char c4, final int SIZE)
   {
      super(currentIndex, c1, c2, c3, c4, SIZE); //Initialize a partially filled buffer.


   }


   /**
      Insert the given character at the next available buffer location.

      Every time we do a successful operation on this buffer, it prints out its state.

      @param c  character to put in the buffer
      @param time  amount of time to wait for a free space in this buffer
      @param unit  the {@link TimeUnit} of time to use in the wait
      @return true if the char c was placed in the buffer, false if the buffer timed out
   */
   public boolean put(final char c, final long time, final TimeUnit unit)
   {
      try
      {

      }
      catch (InterruptedException e)
      {
         // Without this line, the program can deadlock.
         Thread.currentThread().interrupt();
      }
      return false; // keep the compiler from complaining
   }


   /**
      Remove from the buffer the character at the buffer's current index.
      Mark the newly emptied spot in the buffer with the given character.
      Return, in an Optional object, the character that was removed from the buffer.
      If the buffer times out, return an "empty" Optional object.

      Every time we do a successful operation on this buffer, it prints out its state.

      @param c  an {@link Optional} character to use to mark the newly emptied position in the buffer
      @param time  amount of time to wait for an item from this buffer
      @param unit  the {@link TimeUnit} of time to use in the wait
      @return an {link Optional} holding the character removed from the buffer
   */
   public Optional<Character> get(final char c, final long time, final TimeUnit unit)
   {
      try
      {

      }
      catch (InterruptedException e)
      {
         // Without this line, the program can deadlock.
         Thread.currentThread().interrupt();
      }
      return Optional.empty(); // keep the compiler from complaining
   }
}
