/*
   Do not modify this file.
*/

import java.util.Random;

/**
   Each time a Consumer should consume an item, it
   randomly chooses a buffer to get an item from.
*/
public class Consumer_v1 implements Runnable
{
   private final Random rn = new Random();
   private final Buffer[] buf;
   private final char c;
   private int ms;

   /**
      Create a Consumer that uses the given array of bounded buffers.
      The consumer will randomly choose a buffer from the array
      each time the consumer consumes an item.

      After successfully getting an item from a buffer, the consumer
      should sleep for the given number of milliseconds.

      @param buf  array of {@link Buffer} objects
      @param c    character to put into a buffer in place of a consumed character
      @param ms   milliseconds of delay after consuming one item
   */
   public Consumer_v1(final Buffer[] buf, final char c, final int ms)
   {
      this.buf = buf;
      this.c = c;
      this.ms = ms;
   }


   @Override
   public void run()
   {
      try
      {
         while(! Thread.currentThread().isInterrupted())
         {
            int i = rn.nextInt( buf.length );
            buf[i].get(c);
            Thread.sleep(ms);
         }
      }
      catch (InterruptedException e)
      {
      }
   }
}
