/*
   Do not modify this file.
*/

import java.util.Random;

/**
   Each time a Producer should produce an item, it
   randomly chooses a buffer to put the item into.
*/
public class Producer_v1 implements Runnable
{
   private final Random rn = new Random();
   private final Buffer[] buf;
   private final char c;
   private int ms;

   /**
      Create a Producer that uses the given array of bounded buffers.
      The producer will randomly choose a buffer from the array
      each time the producer produces an item.

      After successfully putting an item in a buffer, the producer
      should sleep for the given number of milliseconds.

      @param buf  array of {@link Buffer} objects
      @param c    character to place into a buffer
      @param ms   milliseconds of delay after producing one item
   */
   public Producer_v1(final Buffer[] buf, final char c, final int ms)
   {
      this.buf = buf;
      this.c = c;
      this.ms = ms;
   }


   @Override
   public void run()
   {
      try
      {
         while(! Thread.currentThread().isInterrupted())
         {
            int i = rn.nextInt( buf.length );
            buf[i].put(c);
            Thread.sleep(ms);
         }
      }
      catch (InterruptedException e)
      {
      }
   }
}
